/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

import ClayButton from '@clayui/button';
import ClayDropDown from '@clayui/drop-down';
import ClayForm from '@clayui/form';
import ClayIcon from '@clayui/icon';
import PropTypes from 'prop-types';
import React, {useCallback, useMemo, useState} from 'react';

import ImportMappingDropdownItem, {
	ImportFieldPropType,
} from './ImportMappingDropdownItem';

const ImportMappingItem = ({
	field,
	onChange,
	portletNamespace,
	selectableFields,
	selectedField,
}) => {
	const [searchLabel, setSearchLabel] = useState();
	const [dropDownActive, setDropDownActive] = useState(false);

	const [requiredFields, optionalFields] = useMemo(
		() =>
			buildDropdownItemsFromFields(
				selectableFields,
				searchLabel,
				selectedField
			),
		[selectableFields, searchLabel, selectedField]
	);

	const inputId = `input-field-${field}`;

	const onSearchChange = useCallback((event) => {
		setSearchLabel(event.target.value);
	}, []);

	const onDropdownItemClick = useCallback(
		(item) => {
			if (onChange) {
				onChange(item, field);
			}
			setDropDownActive(false);
		},
		[field, onChange]
	);

	const selectFirstElement = useCallback(
		(event) => {
			event.preventDefault();
			const firstElement = requiredFields?.at(0) || optionalFields?.at(0);

			if (firstElement) {
				if (onChange) {
					onChange(firstElement, field);
				}
				setDropDownActive(false);
				setSearchLabel();
			}
		},
		[field, onChange, optionalFields, requiredFields]
	);

	return (
		<ClayForm.Group>
			<label htmlFor={inputId}>{field}</label>

			<input
				hidden
				name={`${portletNamespace}externalFieldName_${selectedField?.value}`}
				readOnly
				value={field}
			/>

			<input
				hidden
				name={`${portletNamespace}internalFieldName_${selectedField?.value}`}
				readOnly
				value={selectedField?.value}
			/>

			<ClayDropDown
				active={dropDownActive}
				onActiveChange={setDropDownActive}
				tabIndex={-1}
				trigger={
					<ClayButton
						aria-expanded={dropDownActive ? 'true' : 'false'}
						aria-haspopup="listbox"
						className="w-100"
						displayType="secondary"
						id={inputId}
					>
						<span className="align-items-center d-flex justify-content-between">
							<span>{selectedField?.label ?? '\u00A0'}</span>

							<ClayIcon symbol="caret-double" />
						</span>
					</ClayButton>
				}
			>
				<ClayDropDown.Search
					formProps={{onSubmit: selectFirstElement}}
					onChange={onSearchChange}
					placeholder={Liferay.Language.get('search')}
					value={searchLabel}
				/>

				<ClayDropDown.ItemList>
					{requiredFields.length > 0 && (
						<ClayDropDown.Group
							header={Liferay.Language.get('required')}
						>
							<ClayDropDown.Divider />

							{requiredFields.map((item) => (
								<ImportMappingDropdownItem
									item={item}
									key={item.value}
									onClick={onDropdownItemClick}
									selectedItem={selectedField}
								/>
							))}
						</ClayDropDown.Group>
					)}

					{optionalFields.length > 0 && (
						<ClayDropDown.Group
							header={Liferay.Language.get('optional')}
						>
							<ClayDropDown.Divider />

							{optionalFields.map((item) => (
								<ImportMappingDropdownItem
									item={item}
									key={item.value}
									onClick={onDropdownItemClick}
									selectedItem={selectedField}
								/>
							))}
						</ClayDropDown.Group>
					)}
				</ClayDropDown.ItemList>
			</ClayDropDown>
		</ClayForm.Group>
	);
};

const buildDropdownItemsFromFields = (
	selectableFields = [],
	searchLabel,
	selectedField
) => {
	const allFields = [...selectableFields];

	if (selectedField) {
		allFields.push(selectedField);
		allFields.sort((a, b) => (a.label > b.label ? 1 : -1));
	}

	const searchedFields = allFields.filter((fields) =>
		searchLabel
			? fields.label.toLowerCase().includes(searchLabel.toLowerCase())
			: true
	);

	const {optionalFields, requiredFields} = searchedFields.reduce(
		(accumulator, currentField) => {
			if (currentField.required) {
				accumulator.requiredFields.push(currentField);
			}
			else {
				accumulator.optionalFields.push(currentField);
			}

			return accumulator;
		},
		{optionalFields: [], requiredFields: []}
	);

	return [requiredFields, optionalFields];
};

ImportMappingItem.propTypes = {
	field: PropTypes.string.isRequired,
	onChange: PropTypes.func,
	portletNamespace: PropTypes.string.isRequired,
	selectableFields: PropTypes.arrayOf(PropTypes.shape(ImportFieldPropType)),
	selectedField: PropTypes.shape(ImportFieldPropType),
};

export default ImportMappingItem;
